"""
   Copyright 2009 Glenn Johnson

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import datetime, wx, wx.html, re
from wx.lib.pubsub import Publisher
from pkg_resources import resource_filename
import dialogs, render

class DetailFrame(wx.Frame):
    def __init__(self, db, recs_html, config, history):
	self.history=history
	config.SetPath('/Frames')
	width = config.ReadInt('detail_width', 700)
	height = config.ReadInt('detail_height', 600)
	sash = config.ReadInt('detail_sash', 0)
	wx.Frame.__init__(self, None, -1,
		"Portfolio Details",
		size=(width, height))
	self.panel = wx.Panel(self, -1)
	self.split = wx.SplitterWindow(self.panel)
	self.panel1 = wx.Panel(self.split, -1)
	self.panel2 = wx.Panel(self.split, -1)
	self.split.SplitHorizontally(self.panel1, self.panel2, sash)
	self.split.SetMinimumPaneSize(20)

	iconFile = resource_filename(__name__, 'icons/bar_chart.ico')
	ib = wx.IconBundle()
	ib.AddIconFromFile(iconFile, wx.BITMAP_TYPE_ANY)
	self.SetIcons(ib)

	D=db['portfolios']
	cash=db['cash']
	label = wx.StaticText(self.panel, -1,
		"Portfolio symbol")
	self.choice = wx.Choice(self.panel, -1, choices=sorted(D.keys()))
	self.choice.SetSelection(0)
	self.edit_btn = wx.Button(self.panel, -1, "&Edit")
	self.close_btn = wx.Button(self.panel, -1, "&Close")
	self.back_btn = wx.Button(self.panel, wx.ID_BACKWARD)
	self.next_btn = wx.Button(self.panel, wx.ID_FORWARD)

	self.details_html = dialogs.MyHtmlWindow(self.panel1, -1)
	if "gtk2" in wx.PlatformInfo:
	    self.details_html.SetStandardFonts()
	self.details_html.SetRelatedFrame(self, self.GetTitle() + ": %s")
	self.details_html.SetRelatedStatusBar(2)

	self.statusbar = self.CreateStatusBar(number=3)

	symbol = sorted(D.keys())[0]
	stock = D[symbol]
	self.render = render.Render()
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)
	
	self.log = wx.TextCtrl(self.panel2, -1,
		style=wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL)

	sizer = wx.BoxSizer(wx.VERTICAL)
	vslot1 = wx.FlexGridSizer(rows=1, cols=6, hgap=5, vgap=5)
	vslot1.AddMany(
		[(label, 0, wx.ALIGN_CENTER), 
		    (self.choice, 0, wx.ALIGN_CENTER),
		    (self.back_btn, 0, wx.ALIGN_CENTER),
		    (self.next_btn, 0, wx.ALIGN_CENTER),
		    (self.edit_btn, 0, wx.ALIGN_CENTER),
		    (self.close_btn, 0, wx.ALIGN_CENTER)])
	sizer.Add(vslot1)
	sizer_2 = wx.BoxSizer(wx.HORIZONTAL)
	sizer.Add(sizer_2, 1, wx.GROW)
	sizer_2.Add(self.split, 1, wx.GROW)
	option_sizer = wx.GridSizer(cols=1)
	sizer_2.Add(option_sizer, 0, wx.GROW)
	param_staticbox = wx.StaticBox(self.panel, -1, "Parameters")
	param_sb_sizer = wx.StaticBoxSizer(param_staticbox, wx.VERTICAL)
	choices = ['All', 'Price Updates', 'Buy', 'Sell', 'Cash In', 
		'Cash Out', 'Capital Gains', 'Dividends', 'Splits']
	self.history_rb = wx.RadioBox(self.panel, -1, "History Display",
		choices=choices, majorDimension=1, style=wx.RA_SPECIFY_COLS)
	option_sizer.Add(param_sb_sizer, 0, wx.ALIGN_TOP)
	option_sizer.Add(self.history_rb, 0, wx.ALIGN_BOTTOM)
	
	p1sizer = wx.BoxSizer(wx.VERTICAL)
	p1sizer.Add(self.details_html, 1, wx.GROW)
	self.panel1.SetSizer(p1sizer)
	p2sizer = wx.BoxSizer(wx.VERTICAL)
	p2sizer.Add(self.log, 1, wx.GROW)
	self.panel2.SetSizer(p2sizer)
	
	br_label = wx.StaticText(self.panel, -1, "Buy Resistance")
	self.br_sc = wx.SpinCtrl(self.panel, -1, initial=stock.buy_resistance)
	
	sr_label = wx.StaticText(self.panel, -1, "Sell Resistance")
	self.sr_sc = wx.SpinCtrl(self.panel, -1, initial=stock.sell_resistance)

	mb_label = wx.StaticText(self.panel, -1, "Minimum Buy")
	self.mb_sc = wx.SpinCtrl(self.panel, -1, initial=stock.min_buy)

	ms_label = wx.StaticText(self.panel, -1, "Minimum Sell")
	self.ms_sc = wx.SpinCtrl(self.panel, -1, initial=stock.min_sell)

	param_sb_sizer.AddMany([(br_label, 0, wx.TOP, 5),
	    (self.br_sc, 0, wx.BOTTOM, 5),
	    sr_label, (self.sr_sc, 0, wx.BOTTOM, 5),
	    mb_label, (self.mb_sc, 0, wx.BOTTOM, 5),
	    ms_label, (self.ms_sc, 0, wx.BOTTOM, 5)])

	self.panel.SetSizer(sizer)
	self.panel.Layout()
	self.SetMinSize(sizer.GetMinSize())
	self.UpdateHistory(symbol)
	self.details_html.SetFocus()

	self.Bind(wx.EVT_BUTTON, self.OnClose, self.close_btn)

	self.Bind(wx.EVT_SPINCTRL, 
		lambda event: self.OnBR(event, db, recs_html), 
		self.br_sc)
	self.Bind(wx.EVT_SPINCTRL, 
		lambda event: self.OnSR(event, db, recs_html),
		self.sr_sc)
	self.Bind(wx.EVT_SPINCTRL, 
		lambda event: self.OnMB(event, db, recs_html),
		self.mb_sc)
	self.Bind(wx.EVT_SPINCTRL, 
		lambda event: self.OnMS(event, db, recs_html),
		self.ms_sc)
	self.Bind(wx.EVT_BUTTON, 
		lambda event: self.OnEdit(event, db, recs_html),
		self.edit_btn)
	self.Bind(wx.EVT_CHOICE, 
		lambda event: self.OnChoice(event, db), 
		self.choice)
	self.Bind(wx.EVT_BUTTON, 
		lambda event: self.OnNext(event, db),
		self.next_btn)
	self.Bind(wx.EVT_BUTTON, 
		lambda event: self.OnBack(event, db),
		self.back_btn)
	self.Bind(wx.EVT_RADIOBOX, 
		lambda event: self.OnRadio(event, symbol), 
		self.history_rb)

	Publisher().subscribe(self.OnReceiveMessage, 'update.cash_level')
	Publisher().subscribe(self.OnReceiveMessage, 'update.prices')
	Publisher().subscribe(self.OnReceiveMessage, 'update.details')
	Publisher().subscribe(self.OnLogMessage, 'update.log')

    def UpdateHistory(self, symbol):
	if self.history_rb.GetStringSelection() == "Price Updates":
	    symbol="Updated price of %s" % symbol
	elif self.history_rb.GetStringSelection() == "Buy":
	    symbol="Bought .* shares of %s" % symbol
	elif self.history_rb.GetStringSelection() == "Sell":
	    symbol="Sold .* shares of %s" % symbol
	elif self.history_rb.GetStringSelection() == "Capital Gains":
	    symbol=("Deposited .* of capital gains distributions into %s" 
		    % symbol)
	elif self.history_rb.GetStringSelection() == "Dividends":
	    symbol=("Deposited .* of dividends into %s" 
		    % symbol)
	elif self.history_rb.GetStringSelection() == "Splits":
	    symbol="Split %s" % symbol
	elif self.history_rb.GetStringSelection() == "Cash Out":
	    symbol="Siphoned .* from %s" % symbol
	elif self.history_rb.GetStringSelection() == "Cash In":
	    symbol="Funded %s" % symbol

	self.log.Clear()
	for line in open(self.history):
	    if re.search(symbol,line):
		self.log.AppendText(line)

    def OnRadio(self, event, symbol):
	self.UpdateHistory(symbol)

    def OnLogMessage(self, message):
	symbol = self.choice.GetStringSelection()
	if re.search(symbol,message.data):
	    # Specify the string here to be the same format as in the file.
	    # This allows the string to be appended to the history display
	    # without having to reparse the file.
	    self.log.AppendText('%s %s\n' %
		    (datetime.datetime.now().strftime('%a, %d %b %Y %H:%M:%S'),
			message.data))

    def OnReceiveMessage(self, message):
	cash = message.data['cash']
	D = message.data['portfolios']
	symbol = self.choice.GetStringSelection()
	stock = D[symbol]
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)

    def OnBR(self, event, db, recs_html):
	cash = db['cash']
	D = db['portfolios']
	symbol = self.choice.GetStringSelection()
	stock = D[symbol]
	stock.buy_resistance = float(self.br_sc.GetValue())
	stock.calc(cash.level, db['delay'])
	db['portfolios'] = D
	db.sync()
	self.render.recs(recs_html, db)
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)

    def OnSR(self, event, db, recs_html):
	cash = db['cash']
	D = db['portfolios']
	symbol = self.choice.GetStringSelection()
	stock = D[symbol]
	stock.sell_resistance = float(self.sr_sc.GetValue())
	stock.calc(cash.level, db['delay'])
	db['portfolios'] = D
	db.sync()
	self.render.recs(recs_html, db)
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)
    
    def OnMB(self, event, db, recs_html):
	cash = db['cash']
	D = db['portfolios']
	symbol = self.choice.GetStringSelection()
	stock = D[symbol]
	stock.min_buy = float(self.mb_sc.GetValue())
	stock.calc(cash.level, db['delay'])
	db['portfolios'] = D
	db.sync()
	self.render.recs(recs_html, db)
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)
    
    def OnMS(self, event, db, recs_html):
	cash = db['cash']
	D = db['portfolios']
	symbol = self.choice.GetStringSelection()
	stock = D[symbol]
	stock.min_sell = float(self.ms_sc.GetValue())
	stock.calc(cash.level, db['delay'])
	db['portfolios'] = D
	db.sync()
	self.render.recs(recs_html, db)
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)

    def OnEdit(self, event, db, recs_html):
	cash = db['cash']
	D = db['portfolios']
	symbol = self.choice.GetStringSelection()
	stock = D[symbol]
	edit_d = dialogs.Edit(self, -1, 'Edit', stock)
	edit_d.shares_numctrl.SetMin(0.000001)
	edit_d.shares_numctrl.SetLimited(True)
	result = edit_d.ShowModal()
	if result == wx.ID_OK:
	    stock.symbol = edit_d.ticker_textctrl.GetValue()
	    stock.description = edit_d.descript_textctrl.GetValue()
	    stock.shares = edit_d.shares_numctrl.GetValue()
	    stock.portfolio_control = edit_d.cpc_numctrl.GetValue()
	    stock.buy_portfolio_control = edit_d.bpc_numctrl.GetValue()
	    stock.sell_portfolio_control = edit_d.spc_numctrl.GetValue()
	    stock.calc(cash.level, db['delay'])
	    db['portfolios'] = D
	    db.sync()
	    edit_d.Destroy()
	    self.render.recs(recs_html, db)
	    self.render.details(self.details_html, stock, cash)
	    self.statusbar.SetStatusText(
		    "Buy Resistance (%%): %.2f" 
		    % stock.scaled_buy_resistance, 0)
	    self.statusbar.SetStatusText(
		    "Sell Resistance (%%): %.2f"
		    % stock.scaled_sell_resistance, 1)
	    return
	else:
	    edit_d.Destroy()

    def OnNext(self, event, db):
	cash = db['cash']
	D = db['portfolios']
	count=self.choice.GetCount()
	index=self.choice.GetSelection()
	if index < (count - 1):
	    index += 1
	else:
	    return
	self.choice.SetSelection(index)
	symbol = self.choice.GetString(index)
	stock = D[symbol]
	self.br_sc.SetValue(stock.buy_resistance)
	self.sr_sc.SetValue(stock.sell_resistance)
	self.mb_sc.SetValue(stock.min_buy)
	self.ms_sc.SetValue(stock.min_sell)
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)
	self.UpdateHistory(symbol)

    def OnBack(self, event, db):
	cash = db['cash']
	D = db['portfolios']
	count=self.choice.GetCount()
	index=self.choice.GetSelection()
	if index > 0:
	    index -= 1
	else:
	    return
	self.choice.SetSelection(index)
	symbol = self.choice.GetString(index)
	stock = D[symbol]
	self.br_sc.SetValue(stock.buy_resistance)
	self.sr_sc.SetValue(stock.sell_resistance)
	self.mb_sc.SetValue(stock.min_buy)
	self.ms_sc.SetValue(stock.min_sell)
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)
	self.UpdateHistory(symbol)

    def OnClose(self, event):
	width, height = self.GetSize()
	sash = self.split.GetSashPosition()
	Publisher().sendMessage('details.size', (width, height, sash))
	self.Close()

    def OnChoice(self, event, db):
	cash = db['cash']
	D = db['portfolios']
	symbol = event.GetString()
	stock = D[symbol]
	self.br_sc.SetValue(stock.buy_resistance)
	self.sr_sc.SetValue(stock.sell_resistance)
	self.mb_sc.SetValue(stock.min_buy)
	self.ms_sc.SetValue(stock.min_sell)
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)
	self.UpdateHistory(symbol)

