"""
   Copyright 2009 Glenn Johnson

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import wx, wx.calendar
from wx.lib.masked import NumCtrl

class NotEmptyValidator(wx.PyValidator):
    def __init__(self):
	wx.PyValidator.__init__(self)

    def Clone(self):
	return NotEmptyValidator()

    def Validate(self, win):
	textCtrl = self.GetWindow()
	text = textCtrl.GetValue()

	if len(text) == 0:
	    wx.MessageBox("This field must contain a value!", "Error",
		    wx.ICON_ERROR)
	    textCtrl.SetBackgroundColour("yellow")
	    textCtrl.SetFocus()
	    textCtrl.Refresh()
	    return False
	else:
	    textCtrl.SetBackgroundColour(
		    wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
	    textCtrl.Refresh()
	    return True

    def TransferToWindow(self):
	    return True

    def TransferFromWindow(self):
	    return True

class DepositWithdraw(wx.Dialog):
    def __init__(self, title, label):
	wx.Dialog.__init__(self, None, -1, title)
	vbox = wx.BoxSizer(wx.VERTICAL)
	money_label = wx.StaticText(self, -1,
		label)
	money_label.SetForegroundColour("Blue")
	self.money_numctrl = NumCtrl(self, -1, 
		fractionWidth=2, selectOnEntry=False)
	vbox.Add(money_label)
	vbox.Add(self.money_numctrl)
	sizer = self.CreateSeparatedButtonSizer(wx.OK | wx.CANCEL)
	vbox.Add(sizer, 1, wx.GROW)
	self.SetSizer(vbox)
	self.Fit()
	self.money_numctrl.SetFocus()

class NewPort(wx.Dialog):
    def __init__(self, mlabel):
	wx.Dialog.__init__(self, None, -1, "New Portfolio")
	vbox = wx.BoxSizer(wx.VERTICAL)
	vslot1 = wx.BoxSizer(wx.VERTICAL)
	vbox.Add(vslot1, 1, wx.GROW)
	vslot2 = wx.FlexGridSizer(rows=4, cols=2, hgap=5, vgap=5)
	vbox.Add(vslot2)
	label = wx.StaticText(self, -1, 
		"Enter ticker symbol, description, funding, and parameters")
	vslot1.Add(label, 1, wx.ALIGN_CENTER | wx.BOTTOM, 20)
	label.SetForegroundColour("Blue")

	ticker_label = wx.StaticText(self, -1,
		"Enter ticker symbol")
	self.ticker_textctrl = wx.TextCtrl(self, -1,
		validator=NotEmptyValidator()) 
	descript_label = wx.StaticText(self, -1,
		"Enter description")
	self.descript_textctrl = wx.TextCtrl(self, -1, 
		size=(200,-1), validator=NotEmptyValidator())
	money_label = wx.StaticText(self, -1,
		mlabel)
	self.money_numctrl = NumCtrl(self, -1, 
		fractionWidth=2, selectOnEntry=False)

	res_staticbox = wx.StaticBox(self, -1, "Resistance")
	res_sb_sizer = wx.StaticBoxSizer(res_staticbox, wx.VERTICAL)
	br_label = wx.StaticText(self, -1, "Buy")
	self.br_sc = wx.SpinCtrl(self, -1, initial=5)
	sr_label = wx.StaticText(self, -1, "Sell")
	self.sr_sc = wx.SpinCtrl(self, -1, initial=5)
	res_sb_sizer.AddMany([(br_label, 0, wx.TOP, 5),
	    (self.br_sc, 0, wx.BOTTOM, 5),
	    sr_label, (self.sr_sc, 0, wx.BOTTOM, 5)])

	min_staticbox = wx.StaticBox(self, -1, "Minimum")
	min_sb_sizer = wx.StaticBoxSizer(min_staticbox, wx.VERTICAL)
	mb_label = wx.StaticText(self, -1, "Buy")
	self.mb_sc = wx.SpinCtrl(self, -1, initial=5)
	ms_label = wx.StaticText(self, -1, "Sell")
	self.ms_sc = wx.SpinCtrl(self, -1, initial=5)
	min_sb_sizer.AddMany([(mb_label, 0, wx.TOP, 5),
	    (self.mb_sc, 0, wx.BOTTOM, 5),
	    ms_label, (self.ms_sc, 0, wx.BOTTOM, 5)])

	vslot2.AddMany([ticker_label, self.ticker_textctrl, 
	    descript_label, self.descript_textctrl, 
	    money_label, self.money_numctrl, 
	    res_sb_sizer, min_sb_sizer])
	
	sizer = self.CreateSeparatedButtonSizer(wx.OK | wx.CANCEL)
	vbox.Add(sizer, 1, wx.GROW)
	self.SetSizer(vbox)
	self.Fit()
	self.ticker_textctrl.SetFocus()

class BuySell(wx.Dialog):
    def __init__(self, title, symbol, price):
	wx.Dialog.__init__(self, None, -1, title)
	vbox = wx.BoxSizer(wx.VERTICAL)
	vslot1 = wx.BoxSizer(wx.VERTICAL)
	vbox.Add(vslot1, 1, wx.GROW)
	vslot2 = wx.FlexGridSizer(rows=4, cols=2, hgap=5, vgap=5)
	vbox.Add(vslot2)
	label = wx.StaticText(self, -1, 
		"Transaction parameters for %s" % symbol.upper())
	vslot1.Add(label, 1, wx.ALIGN_CENTER | wx.BOTTOM, 20)
	label.SetForegroundColour("Blue")
	date_label = wx.StaticText(self, -1,
		"Transaction date ")
	self.cal = wx.calendar.CalendarCtrl(self, -1, wx.DateTime_Now(),
		style=wx.calendar.CAL_SEQUENTIAL_MONTH_SELECTION)
	vslot2.Add(date_label)
	vslot2.Add(self.cal)
	
	shares_label = wx.StaticText(self, -1,
		"Number of shares ")
	self.shares_numctrl = NumCtrl(self, -1, 
		fractionWidth=6, selectOnEntry=False)
	vslot2.Add(shares_label)
	vslot2.Add(self.shares_numctrl)
	
	price_label = wx.StaticText(self, -1,
		"Price per share ")
	self.price_numctrl = NumCtrl(self, -1, 
		value=price, fractionWidth=6, selectOnEntry=True)
	vslot2.Add(price_label)
	vslot2.Add(self.price_numctrl)

	commision_label = wx.StaticText(self, -1,
		"Commission ")
	self.commission_numctrl = NumCtrl(self, -1, 
		fractionWidth=2, selectOnEntry=False)
	vslot2.Add(commision_label)
	vslot2.Add(self.commission_numctrl)
	sizer = self.CreateSeparatedButtonSizer(wx.OK | wx.CANCEL)
	vbox.Add(sizer, 0, wx.GROW)
	self.SetSizer(vbox)
	self.Fit()
	self.shares_numctrl.SetFocus()

class Price(wx.Dialog):
    def __init__(self, title, label):
	wx.Dialog.__init__(self, None, -1, title)
	vbox = wx.BoxSizer(wx.VERTICAL)
	price_label = wx.StaticText(self, -1,
		label)
	price_label.SetForegroundColour("Blue")
	self.price_numctrl = NumCtrl(self, -1, 
		fractionWidth=6, selectOnEntry=False)
	vbox.Add(price_label)
	vbox.Add(self.price_numctrl)
	sizer = self.CreateSeparatedButtonSizer(wx.OK | wx.CANCEL)
	vbox.Add(sizer, 1, wx.GROW)
	self.SetSizer(vbox)
	self.Fit()
	self.price_numctrl.SetFocus()

class Split(wx.Dialog):
    def __init__(self, title, label):
	wx.Dialog.__init__(self, None, -1, title)
	vbox = wx.BoxSizer(wx.VERTICAL)
	split_label = wx.StaticText(self, -1,
		label)
	split_label.SetForegroundColour("Blue")
	vbox.Add(split_label)
	vslot2 = wx.FlexGridSizer(rows=1, cols=3, hgap=5, vgap=5)
	self.split_newShares = wx.SpinCtrl(self, -1, min=1)
	delim = wx.StaticText(self, -1, "for")
	self.split_oldShares = wx.SpinCtrl(self, -1, min=1)
	vslot2.AddMany([self.split_newShares, delim, self.split_oldShares]) 
	vbox.Add(vslot2)
	sizer = self.CreateSeparatedButtonSizer(wx.OK | wx.CANCEL)
	vbox.Add(sizer, 1, wx.GROW)
	self.SetSizer(vbox)
	self.Fit()
	self.split_newShares.SetFocus()

class Edit(wx.Dialog):
    def __init__(self, parent, ID, title, stock):
	wx.Dialog.__init__(self, parent, ID, title)
	data_symbol = {stock.symbol : stock.symbol}
	data_description = {stock.symbol : stock.description}
	vbox = wx.BoxSizer(wx.VERTICAL)
	vslot1 = wx.FlexGridSizer(rows=4, cols=2, hgap=5, vgap=5)
	vbox.Add(vslot1)
	ticker_label = wx.StaticText(self, -1, "Symbol")
	self.ticker_textctrl = wx.TextCtrl(self, -1, stock.symbol)
	descript_label = wx.StaticText(self, -1, "Description")
	self.descript_textctrl = wx.TextCtrl(self, -1, stock.description,
		size=(400, -1))
	shares_label = wx.StaticText(self, -1, "Shares")
	self.shares_numctrl = NumCtrl(self, -1, stock.shares,
		fractionWidth=6, selectOnEntry=False)
	cpc_label = wx.StaticText(self, -1, "Core Portfolio Control")
	self.cpc_numctrl = NumCtrl(self, -1, stock.portfolio_control,
		fractionWidth=6, selectOnEntry=True)
	bpc_label = wx.StaticText(self, -1, "Buy Portfolio Control")
	self.bpc_numctrl = NumCtrl(self, -1, stock.buy_portfolio_control,
		fractionWidth=6, selectOnEntry=True)
	spc_label = wx.StaticText(self, -1, "Sell Portfolio Control")
	self.spc_numctrl = NumCtrl(self, -1, stock.sell_portfolio_control,
		fractionWidth=6, selectOnEntry=True)
	vslot1.AddMany([ticker_label, self.ticker_textctrl,
	    descript_label, self.descript_textctrl,
	    shares_label, self.shares_numctrl,
	    cpc_label, self.cpc_numctrl,
	    bpc_label, self.bpc_numctrl,
	    spc_label, self.spc_numctrl])

	sizer = self.CreateSeparatedButtonSizer(wx.OK|wx.CANCEL)
	vbox.Add(sizer, 1, wx.GROW)
	self.SetSizer(vbox)
	self.Fit()
	self.ticker_textctrl.SetFocus()

class MyHtmlWindow(wx.html.HtmlWindow):
    def __init__(self, parent, id):
	wx.html.HtmlWindow.__init__(self, parent, id)

    def OnLinkClicked(self, link):
	wx.LaunchDefaultBrowser(link.GetHref())

